
\name{e04kd}
\alias{e04kd}

\title{e04kd: Minimum, function of several variables, modified Newton algorithm, simple 
bounds, using first derivatives (comprehensive)}

\description{
e04kd is a comprehensive modified Newton algorithm for finding: 

- an unconstrained minimum of a function of several variables;

- a minimum of a function of several variables subject to fixed upper and/or 
lower bounds on the variables.



First derivatives are required. The function is intended for functions 
which have continuous first and second derivatives (although it will usually 
work even if the derivatives have occasional discontinuities).

}

\usage{
e04kd(funct, monit, eta, ibound, bl, bu, x, lh, iw, w,
n=nrow(bl),
iprint=1,
maxcal=50,
xtol=0.0,
delta=0.0,
stepmx=100000.0)
}

\arguments{


\item{funct}{void function

funct must evaluate the function \eqn{{F}({x})}{F(x)} and its first derivatives \eqn{\frac{{ \partial {F}}}{{ \partial {x}_{{j}}}}}{( \partial F)/( \partial x_j)} at a specified point.  (However, if you do not wish to calculate \eqn{{F}}{F} or its first derivatives at a particular \eqn{{x}}{x}, there is the option of setting a argument to cause e04kd to terminate immediately.)
}
\item{monit}{void function

If \eqn{iprint \geq {0}}{iprint  >=  0}, you must supply monit which is suitable for monitoring the minimization process.  monit must not change the values of any of its arguments.
}

\item{eta}{double

Every iteration of e04kd involves a linear minimization (i.e., minimization 
of \eqn{{F}({x} + {\alpha}{p})}{F(x + \alphap)} with respect to \eqn{{\alpha}}{\alpha}). eta 
specifies how accurately these linear minimizations are to be performed. 
The minimum with respect to \eqn{{\alpha}}{\alpha} will be located more accurately 
for small values of eta (say, \eqn{{0.01}}{0.01}) than large values (say, 
\eqn{{0.9}}{0.9}).


}

\item{ibound}{integer

Indicates whether the problem is unconstrained or bounded. If there are 
bounds on the variables, ibound can be used to indicate whether the facility 
for dealing with bounds of special forms is to be used. It must be set 
to one of the following values: 

\eqn{ibound = {0}}{ibound = 0}: If the variables are bounded and you are 
supplying all the \eqn{{l}_{{j}}}{l_j} and \eqn{{u}_{{j}}}{u_j} individually.

\eqn{ibound 
= {1}}{ibound = 1}: If the problem is unconstrained.

\eqn{ibound = {2}}{ibound = 2}: If the variables are bounded, but all the 
bounds are of the form \eqn{{0} \leq {x}_{{j}}}{0 <= x_j}.

\eqn{ibound = {3}}{ibound = 3}: If all the variables are bounded, and \eqn{{l}_{{1}} 
= {l}_{{2}} = \cdots = {l}_{{n}}}{l_1 = l_2 = \cdots = l_n} and \eqn{{u}_{{1}} = 
{u}_{{2}} = \cdots = {u}_{{n}}}{u_1 = u_2 = \cdots = u_n}.

\eqn{ibound = {4}}{ibound = 4}: If the problem is unconstrained. (The \eqn{ibound 
= {4}}{ibound = 4} option is provided for consistency with other functions. 
In e04kd it produces the same effect as \eqn{ibound = {1}\mbox{.}}{ibound 
= 1.})




}
\item{bl}{double array

The fixed lower bounds \eqn{{l}_{{j}}}{l_j}.


}
\item{bu}{double array

The fixed upper bounds \eqn{{u}_{{j}}}{u_j}.


}
\item{x}{double array

\eqn{x({j})}{x(j)} must be set to a guess at the \eqn{{j}}{j}th component 
of the position of the minimum for \eqn{{j}={1} \ldots {n}}{j=1 . . . n}.


}
\item{lh}{integer




}
\item{iw}{integer array




}
\item{w}{double array




}
\item{n}{integer: \strong{default} = nrow(bl)

The number \eqn{{n}}{n} of independent variables.


}
\item{iprint}{integer: \strong{default} = 1

The frequency with which monit is to be called. 

\eqn{iprint > {0}}{iprint > 0}: monit is called once every iprint iterations 
and just before exit from e04kd.

\eqn{iprint = {0}}{iprint = 0}: monit is just called at the final point.

 
\eqn{iprint < {0}}{iprint < 0}: monit is not called at all.




}
\item{maxcal}{integer: \strong{default} = 50

The maximum permitted number of evaluations of \eqn{{F}({x})}{F(x)}, i.e., 
the maximum permitted number of calls of funct with iflag set to \eqn{{2}}{2}. 
It should be borne in mind that, in addition to the calls of funct which 
are limited directly by maxcal, there will be calls of funct (with iflag 
set to \eqn{{1}}{1}) to evaluate only first derivatives.


}
\item{xtol}{double: \strong{default} = 0.0

The accuracy in \eqn{{x}}{x} to which the solution is required.


}
\item{delta}{double: \strong{default} = 0.0

The differencing interval to be used for approximating the second derivatives 
of \eqn{{F}({x})}{F(x)}. Thus, for the finite difference approximations, 
the first derivatives of \eqn{{F}({x})}{F(x)} are evaluated at points which 
are delta apart. If \eqn{{\epsilon}}{\epsilon} is the machine precision, then \eqn{\sqrt{{\epsilon}}}{sqrt(\epsilon)} 
will usually be a suitable setting for delta. If you set delta to \eqn{{0.0}}{0.0} 
(or to any positive value less than \eqn{{\epsilon}}{\epsilon}), e04kd will automatically 
use \eqn{\sqrt{{\epsilon}}}{sqrt(\epsilon)} as the differencing interval.


}
\item{stepmx}{double: \strong{default} = 100000.0

An estimate of the Euclidean distance between the solution and the starting 
point supplied by you. (For maximum efficiency a slight overestimate is 
preferable.)


}
}

\details{
R interface to the NAG Fortran routine E04KDF.}

\value{

\item{bl}{double array

The lower bounds actually used by e04kd, e.g., If \eqn{ibound = {2}}{ibound 
= 2}, \eqn{bl({1}) = bl({2}) = \cdots = bl({n}) = {0.0}}{bl(1) = bl(2) = \cdots = 
bl(n) = 0.0}.


}
\item{bu}{double array

The upper bounds actually used by e04kd, e.g., if \eqn{ibound = {2}}{ibound 
= 2}, \eqn{bu({1}) = bu({2}) = \cdots = bu({n}) = {10}^{{6}}}{bu(1) = bu(2) 
= \cdots = bu(n) = 10^6}.


}
\item{x}{double array

The final point \eqn{{x}^{({k})}}{x^(k)}. Thus, if \eqn{\mathrm{ifail} 
=0}{ifail =0} on exit, \eqn{x({j})}{x(j)} is the \eqn{{j}}{j}th component 
of the estimated position of the minimum.


}
\item{hesl}{double array

During the determination of a direction \eqn{{p}_{{z}}}{p_z} (see the Description 
in Fortran library documentation), \eqn{{H} + {E}}{H + E} is decomposed 
into the product \eqn{{L}{D}{L}^{T}}{LDL^T}, where \eqn{{L}}{L} is a unit 
lower triangular matrix and \eqn{{D}}{D} is a diagonal matrix. (The matrices 
\eqn{{H}}{H}, \eqn{{E}}{E}, \eqn{{L}}{L} and \eqn{{D}}{D} are all of dimension 
\eqn{{n}_{{z}}}{n_z}, where \eqn{{n}_{{z}}}{n_z} is the number of variables 
free from their bounds. \eqn{{H}}{H} consists of those rows and columns 
of the full estimated second derivative matrix which relate to free variables. 
\eqn{{E}}{E} is chosen so that \eqn{{H} + {E}}{H + E} is positive definite.)


}
\item{hesd}{double array

During the determination of a direction \eqn{{p}_{{z}}}{p_z} (see the Description 
in Fortran library documentation), \eqn{{H} + {E}}{H + E} is decomposed 
into the product \eqn{{L}{D}{L}^{T}}{LDL^T}, where \eqn{{L}}{L} is a unit 
lower triangular matrix and \eqn{{D}}{D} is a diagonal matrix. (The matrices 
\eqn{{H}}{H}, \eqn{{E}}{E}, \eqn{{L}}{L} and \eqn{{D}}{D} are all of dimension 
\eqn{{n}_{{z}}}{n_z}, where \eqn{{n}_{{z}}}{n_z} is the number of variables 
free from their bounds. \eqn{{H}}{H} consists of those rows and columns 
of the full estimated second derivative matrix which relate to free variables. 
\eqn{{E}}{E} is chosen so that \eqn{{H} + {E}}{H + E} is positive definite.)


}
\item{istate}{integer array

Information about which variables are currently on their bounds and which 
are free. If \eqn{istate({j})}{istate(j)} is: 

- equal to \eqn{{ - {1}}}{ - 1}, \eqn{{x}_{{j}}}{x_j} is fixed on its upper 
bound;

- equal to \eqn{{ - {2}}}{ - 2}, \eqn{{x}_{{j}}}{x_j} is fixed on its lower 
bound;

- equal to \eqn{{ - {3}}}{ - 3}, \eqn{{x}_{{j}}}{x_j} is effectively a 
constant (i.e., \eqn{{l}_{{j}} = {u}_{{j}}}{l_j = u_j});

- positive, \eqn{istate({j})}{istate(j)} gives the position of \eqn{{x}_{{j}}}{x_j} 
in the sequence of free variables.




}
\item{f}{double

The function value at the final point given in x.


}
\item{g}{double array

The first derivative vector corresponding to the final point given in x. 
The components of g corresponding to free variables should normally be 
close to zero.


}}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04kdf.pdf}
}

\author{NAG}

\examples{
e04kd_funct = function(iflag, n, xc, fc, gc) {
    
    gc <- as.matrix(mat.or.vec(n, 1))
    fc <- 0
    
    if (iflag != 1) {
        
        fc <- (xc[1] + 10 * xc[2])^2 + 5 * (xc[3] - xc[4])^2 + 
            (xc[2] - 2 * xc[3])^4 + 10 * (xc[1] - xc[4])^4
    }
    gc[1] <- 2 * (xc[1] + 10 * xc[2]) + 40 * (xc[1] - xc[4])^3
    gc[2] <- 20 * (xc[1] + 10 * xc[2]) + 4 * (xc[2] - 2 * xc[3])^3
    gc[3] <- 10 * (xc[3] - xc[4]) - 8 * (xc[2] - 2 * xc[3])^3
    gc[4] <- 10 * (xc[4] - xc[3]) - 40 * (xc[1] - xc[4])^3
    list(IFLAG = iflag, FC = fc, GC = as.matrix(gc))
}
e04kd_monit = function(n, xc, fc, gc, istate, gpjnrm, 
    cond, posdef, niter, nf) {
    
    sprintf("\n Itn Fn evals Fn value Norm of proj gradient\n", 
        "\n")
    
    sprintf(" \%3d \%5d \%20.4f \%20.4f\n", niter, nf, fc, gpjnrm, 
        "\n")
    
    sprintf("\n J XJ GJ Status\n", "\n")
    
    for (j in c(1:n)) {
        isj <- istate[j]
        if (isj > 0) {
            
            sprintf("\%2d \%16.4f\%20.4f \%s\n", j, xc, j, gc, j, 
                " Free", "\n")
            
        }
        else if (isj == -1) {
            
        }
        else if (isj == -2) {
            
        }
        else if (isj == -3) {
            
        }
    }
    
    if (cond != 0) {
        
        if (cond > 1e+06) {
            
            sprintf("\nEstimated condition number of projected Hessian is more than 1.0e+6\n", 
                "\n")
            
        }
        else {
            
            sprintf("\nEstimated condition number of projected Hessian = \%10.2f\n", 
                cond, "\n")
            
        }
        if (!posdef) {
            
            sprintf("\nProjected Hessian matrix is not positive definite\n", 
                "\n")
            
        }
    }
    list()
}

eta <- 0.5
ibound <- 0
bl <- matrix(c(1, -2, -1e+06, 1), nrow = 4, ncol = 1, 
    byrow = TRUE)


bu <- matrix(c(3, 0, 1e+06, 3), nrow = 4, ncol = 1, 
    byrow = TRUE)


x <- matrix(c(3, -1, 0, 1), nrow = 4, ncol = 1, byrow = TRUE)


lh <- 6
iw <- matrix(c(0, 0), nrow = 2, ncol = 1, byrow = TRUE)


w <- as.matrix(mat.or.vec(34, 1))
e04kd(e04kd_funct, e04kd_monit, eta, ibound, bl, bu, 
    x, lh, iw, w) 

}

\keyword{math}

\keyword{optimize}
\concept{bound}
\concept{derivative}
\concept{minimum}
\concept{modified}
\concept{Newton}
