
\name{e04mf}
\alias{e04mf}

\title{e04mf: LP problem (dense)}

\description{
e04mf solves general linear programming problems. It is not intended for 
large sparse problems.


}

\usage{
e04mf(a, bl, bu, cvec, istate, x, optlist,
      n = nrow(x),
      nclin = nrow(a))
}

\arguments{

\item{a}{double array

The \eqn{{i}}{i}th row of a must contain the coefficients of the \eqn{{i}}{i}th 
general linear constraint for \eqn{{i}={1} \ldots {m}_{{L}}}{i=1  . . .  m_L}.


}
\item{bl}{double array}
\item{bu}{double array

Must contain the lower bounds and bu the upper bounds, for all the constraints 
in the following order. The first \eqn{{n}}{n} elements of each array must 
contain the bounds on the variables, and the next \eqn{{m}_{{L}}}{m_L} 
elements the bounds for the general linear constraints (if any). To specify 
a nonexistent lower bound (i.e., \eqn{{l}_{{j}} = { - {\infty}}}{l_j =  -  infinity}), 
set \eqn{bl[{j}] \leq - {bigbnd}}{bl[j]  <=  - bigbnd}, and to specify a nonexistent 
upper bound (i.e., \eqn{{u}_{{j}} = { + {\infty}}}{u_j =  +  infinity}), set 
\eqn{bu[{j}] \geq {bigbnd}}{bu[j]  >=  bigbnd}; the default value of \eqn{{bigbnd}}{bigbnd} 
is \eqn{{10}^{{20}}}{10^20}, but this may be changed by the optional argument 
infiniteboundsize. To specify the \eqn{{j}}{j}th constraint as an \emph{equality}, 
set \eqn{bl[{j}] = bu[{j}] = {\beta}}{bl[j] = bu[j] = \beta}, say, where \eqn{\mathrm{abs}({\beta}) 
< {bigbnd}}{abs(\beta) < bigbnd}.


}
\item{cvec}{double array

The coefficients of the objective function when the problem is of type 
LP.


}
\item{istate}{integer array

Need not be set if the (default) optional argument coldstart is used.


}
\item{x}{double array

An initial estimate of the solution.


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Check Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {50}}{ = 50} \cr
\code{Cold Start} \tab \emph{} \tab Default \cr
\code{Warm Start} \tab \emph{} \tab  \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.01}}{ = 0.01} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Expand Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {5}}{ = 5} \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Infinite Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}({bigbnd}, {10}^{{20}})}{ = max(bigbnd, 10^20)} \cr
\code{Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{5}\left({n} + {m}_{{L}}\right)})}{ = max(50, 5(n + m_L))} \cr
\code{Iters} \tab \emph{} \tab  \cr
\code{Itns} \tab \emph{} \tab  \cr
\code{List} \tab \emph{} \tab Default for \eqn{e04mf = list}{e04mf = list} \cr
\code{Nolist} \tab \emph{} \tab Default for \eqn{e04mf = nolist}{e04mf = nolist} \cr
\code{Minimum Sum of Infeasibilities} \tab \emph{no} \tab Default \eqn{= {NO}}{= NO} \cr
\code{Monitoring File} \tab \emph{integer} \tab Default \eqn{\mbox{} = { - {1}}}{ =  - 1} \cr
\code{Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.8}}}{ = \epsilon^0.8} \cr
\code{Print Level} \tab \emph{integer} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Problem Type} \tab \emph{string} \tab Default \eqn{=}{=} LP
  \cr
}
}
\item{n}{integer: \strong{default} = nrow(x)

\eqn{{n}}{n}, the number of variables.


}
\item{nclin}{integer: \strong{default} = nrow(a)

\eqn{{m}_{{L}}}{m_L}, the number of general linear constraints.


}
}

\details{
R interface to the NAG Fortran routine E04MFF.}

\value{

\item{ISTATE}{integer array

The status of the constraints in the working set at the point returned 
in x. The significance of each possible value of \eqn{istate[{j}]}{istate[j]} 
is as follows:


}
\item{X}{double array

The point at which e04mf terminated. If \eqn{\mathrm{ifail} =0}{ifail =0}, 
\eqn{\mathrm{ifail} =1}{ifail =1}, \eqn{\mathrm{ifail} =4}{ifail =4}, x 
contains an estimate of the solution.


}
\item{ITER}{integer

The total number of iterations performed.


}
\item{OBJ}{double

The value of the objective function at \eqn{{x}}{x} if \eqn{{x}}{x} is 
feasible, or the sum of infeasibiliites at \eqn{{x}}{x} otherwise. If the 
problem is of type FP and \eqn{{x}}{x} is feasible, obj is set to zero.


}
\item{AX}{double array

The final values of the linear constraints \eqn{{A}{x}}{Ax}.


}
\item{CLAMDA}{double array

The values of the Lagrange multipliers for each constraint with respect 
to the current working set. The first \eqn{{n}}{n} elements contain the 
multipliers for the bound constraints on the variables, and the next \eqn{{m}_{{L}}}{m_L} 
elements contain the multipliers for the general linear constraints (if 
any). If \eqn{istate[{j}] = {0}}{istate[j] = 0} (i.e., constraint \eqn{{j}}{j} 
is not in the working set), \eqn{clamda[{j}]}{clamda[j]} is zero. If \eqn{{x}}{x} 
is optimal, \eqn{clamda[{j}]}{clamda[j]} should be non-negative if \eqn{istate[{j}] 
= {1}}{istate[j] = 1}, non-positive if \eqn{istate[{j}] = {2}}{istate[j] = 2} 
and zero if \eqn{istate[{j}] = {4}}{istate[j] = 4}.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04mff.pdf}
}

\author{NAG}

\examples{

optlist<-list()

ifail<-0

a<-matrix(c(1,1,1,1,1,1,1,0.15,0.04,0.02,0.04,0.02,0.01,0.03,0.03,0.05,0.08,0.02,0.06,0.01,0,0.02,0.04,0.01,0.02,0.02,0,0,0.02,0.03,0,0,0.01,0,0,0.7,0.75,0.8,0.75,0.8,0.97,0,0.02,0.06,0.08,0.12,0.02,0.01,0.97),nrow=7,ncol=7,byrow=TRUE)



bl<-matrix(c(-0.01,-0.1,-0.01,-0.04,-0.1,-0.01,-0.01,-0.13,-9.999999999999999e+24,-9.999999999999999e+24,-9.999999999999999e+24,-9.999999999999999e+24,-0.0992,-0.003),nrow=14,ncol=1,byrow=TRUE)



bu<-matrix(c(0.01,0.15,0.03,0.02,0.05,9.999999999999999e+24,9.999999999999999e+24,-0.13,-0.004900000000000001,-0.0064,-0.0037,-0.0012,9.999999999999999e+24,0.002),nrow=14,ncol=1,byrow=TRUE)



cvec<-matrix(c(-0.02,-0.2,-0.2,-0.2,-0.2,0.04,0.04),nrow=7,ncol=1,byrow=TRUE)



istate<-as.matrix(mat.or.vec(14,1))

x<-matrix(c(-0.01,-0.03,0,-0.01,-0.1,0.02,0.01),nrow=7,ncol=1,byrow=TRUE)



e04mf(a,bl,bu,cvec,istate,x,optlist)

}

\keyword{math}

\keyword{optimize}
\concept{LP}
