
\name{e04us}
\alias{e04us}

\title{e04us: Minimum of a sum of squares, nonlinear constraints, sequential QP method, 
using function values and optionally first derivatives (comprehensive)}

\description{
e04us is designed to minimize an arbitrary smooth sum of squares function 
subject to constraints (which may include simple bounds on the variables, 
linear constraints and smooth nonlinear constraints) using a sequential 
quadratic programming (SQP) method. As many first derivatives as possible 
should be supplied by you; any unspecified derivatives are approximated 
by finite differences. See the description of the optional argument derivativelevel, 
in the Fortran library documentation. It is not intended for large sparse 
problems.

e04us may also be used for unconstrained, bound-constrained and linearly 
constrained optimization.


}

\usage{
e04us(a, bl, bu, y, confun, objfun, istate, cjac, fjac, clamda, r, x, optlist,
      m = nrow(y),
      n = nrow(x),
      nclin = nrow(a),
      ncnln = nrow(cjac))
}

\arguments{

\item{a}{double array

The \eqn{{i}}{i}th row of a contains the \eqn{{i}}{i}th row of the matrix 
\eqn{{A}_{{L}}}{A_L} of general linear constraints in eqn1. That is, the 
\eqn{{i}}{i}th row contains the coefficients of the \eqn{{i}}{i}th general 
linear constraint for \eqn{{i}={1} \ldots nclin}{i=1  . . .  nclin}.


}
\item{bl}{double array}
\item{bu}{double array

Must contain the lower bounds and bu the upper bounds, for all the constraints 
in the following order. The first \eqn{{n}}{n} elements of each array must 
contain the bounds on the variables, the next \eqn{{n}_{{L}}}{n_L} elements 
the bounds for the general linear constraints (if any) and the next \eqn{{n}_{{N}}}{n_N} 
elements the bounds for the general nonlinear constraints (if any). To 
specify a nonexistent lower bound (i.e., \eqn{{l}_{{j}} = { - {\infty}}}{l_j =  -  infinity}), 
set \eqn{bl[{j}] \leq - {bigbnd}}{bl[j]  <=  - bigbnd}, and to specify a nonexistent 
upper bound (i.e., \eqn{{u}_{{j}} = { + {\infty}}}{u_j =  +  infinity}), set 
\eqn{bu[{j}] \geq {bigbnd}}{bu[j]  >=  bigbnd}; the default value of \eqn{{bigbnd}}{bigbnd} 
is \eqn{{10}^{{20}}}{10^20}, but this may be changed by the optional argument 
infiniteboundsize. To specify the \eqn{{j}}{j}th constraint as an \emph{equality}, 
set \eqn{bl[{j}] = bu[{j}] = {\beta}}{bl[j] = bu[j] = \beta}, say, where \eqn{\mathrm{abs}({\beta}) 
< {bigbnd}}{abs(\beta) < bigbnd}.


}
\item{y}{double array

The coefficients of the constant vector \eqn{{y}}{y} of the objective function.


}
\item{confun}{function


confun must calculate the vector \eqn{{c}\left({x}\right)}{c(x)} of nonlinear constraint functions and (optionally) its Jacobian (\eqn{\mbox{} = \frac{{ \partial {c}}}{{ \partial {x}}}}{ = ( \partial c)/( \partial x)}) for a specified \eqn{{n}}{n} element vector \eqn{{x}}{x}.  If there are no nonlinear constraints (i.e., \eqn{ncnln = {0}}{ncnln = 0}), confun will never be called by e04us and confun may be the dummy function e04udm.  (e04udm is included in the NAG Library.) If there are nonlinear constraints, the first call to confun will occur before the first call to objfun.

\code{(MODE,C,CJAC) = confun(mode,ncnln,n,needc,x,cjac,nstate)
}


}
\item{objfun}{function

objfun must calculate either the \eqn{{i}}{i}th element of the vector \eqn{{f}\left({x}\right) = {\left({{f}_{{1}}\left({x}\right)}{{f}_{{2}}\left({x}\right)} \ldots {{f}_{{m}}\left({x}\right)}\right)}^{T}}{f(x) = (f_1(x)f_2(x)  . . .  f_m(x))^T} or all \eqn{{m}}{m} elements of \eqn{{f}\left({x}\right)}{f(x)} and (optionally) its Jacobian (\eqn{\mbox{} = \frac{{ \partial {f}}}{{ \partial {x}}}}{ = ( \partial f)/( \partial x)}) for a specified \eqn{{n}}{n} element vector \eqn{{x}}{x}.

\code{(MODE,F,FJAC) = objfun(mode,m,n,needfi,x,fjac,nstate)
}


}
\item{istate}{integer array

Need not be set if the (default) optional argument coldstart is used.


}
\item{cjac}{double array

In general, cjac need not be initialized before the call to e04us. However, 
if \eqn{derivativelevel={3}}{derivativelevel=3}, you may optionally set 
the constant elements of cjac (see argument nstate in the description 
of confun). Such constant elements need not be re-assigned on subsequent 
calls to confun.


}
\item{fjac}{double array

In general, fjac need not be initialized before the call to e04us. However, 
if \eqn{derivativelevel={3}}{derivativelevel=3}, you may optionally set 
the constant elements of fjac (see argument nstate in the description 
of objfun). Such constant elements need not be re-assigned on subsequent 
calls to objfun.


}
\item{clamda}{double array

Need not be set if the (default) optional argument coldstart is used.


}
\item{r}{double array

Need not be initialized if the (default) optional argument coldstart is 
used.


}
\item{x}{double array

An initial estimate of the solution.


}
\item{optlist}{options list

Optional parameters may be listed, as shown in the following table:

\tabular{lll}{
\bold{Name} \tab \bold{Type} \tab \bold{Default} \cr
\code{Central Difference Interval} \tab \emph{double} \tab Default values are computed \cr
\code{Cold Start} \tab \emph{} \tab Default \cr
\code{Warm Start} \tab \emph{} \tab  \cr
\code{Crash Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.01}}{ = 0.01} \cr
\code{Defaults} \tab \emph{} \tab  \cr
\code{Derivative Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {3}}{ = 3} \cr
\code{Difference Interval} \tab \emph{double} \tab Default values are computed \cr
\code{Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Function Precision} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.9}}}{ = \epsilon^0.9} \cr
\code{Hessian} \tab \emph{no} \tab Default \eqn{= {NO}}{= NO} \cr
\code{Infinite Bound Size} \tab \emph{double} \tab Default \eqn{\mbox{} = {10}^{{20}}}{ = 10^20} \cr
\code{Infinite Step Size} \tab \emph{double} \tab Default \eqn{\mbox{} = \mathrm{max}({bigbnd}, {10}^{{20}})}{ = max(bigbnd, 10^20)} \cr
\code{JTJ Initial Hessian} \tab \emph{} \tab Default \cr
\code{Unit Initial Hessian} \tab \emph{} \tab  \cr
\code{Line Search Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {0.9}}{ = 0.9} \cr
\code{Linear Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = \sqrt{{\epsilon}}}{ = sqrt(\epsilon)} \cr
\code{Nonlinear Feasibility Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}^{{0.33}}}{ = \epsilon^0.33} or \eqn{\sqrt{{\epsilon}}}{sqrt(\epsilon)} \cr
\code{List} \tab \emph{} \tab Default for \eqn{e04us = list}{e04us = list} \cr
\code{Nolist} \tab \emph{} \tab Default for \eqn{e04us = nolist}{e04us = nolist} \cr
\code{Major Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{3}\left({n} + {n}_{{L}}\right) + {10}{n}_{{N}}})}{ = max(50, 3(n + n_L) + 10n_N)} \cr
\code{Iteration Limit} \tab \emph{} \tab  \cr
\code{Iters} \tab \emph{} \tab  \cr
\code{Itns} \tab \emph{} \tab  \cr
\code{Major Print Level} \tab \emph{integer} \tab  \cr
\code{Print Level} \tab \emph{} \tab \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Minor Iteration Limit} \tab \emph{integer} \tab Default \eqn{\mbox{} = \mathrm{max}({50}, {{3}\left({n} + {n}_{{L}} + {n}_{{N}}\right)})}{ = max(50, 3(n + n_L + n_N))} \cr
\code{Minor Print Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Monitoring File} \tab \emph{integer} \tab Default \eqn{\mbox{} = { - {1}}}{ =  - 1} \cr
\code{Optimality Tolerance} \tab \emph{double} \tab Default \eqn{\mbox{} = {\epsilon}_{{R}}^{{0.8}}}{ = \epsilon_R^0.8} \cr
\code{Reset Frequency} \tab \emph{integer} \tab Default \eqn{\mbox{} = {2}}{ = 2} \cr
\code{Start Objective Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Objective Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Start Constraint Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {1}}{ = 1} \cr
\code{Stop Constraint Check At Variable} \tab \emph{integer} \tab Default \eqn{\mbox{} = {n}}{ = n} \cr
\code{Step Limit} \tab \emph{double} \tab Default \eqn{\mbox{} = {2.0}}{ = 2.0} \cr
\code{Verify Level} \tab \emph{integer} \tab Default \eqn{\mbox{} = {0}}{ = 0} \cr
\code{Verify} \tab \emph{} \tab  \cr
\code{Verify Constraint Gradients} \tab \emph{} \tab  \cr
\code{Verify Gradients} \tab \emph{} \tab  \cr
\code{Verify Objective Gradients} \tab \emph{} \tab  \cr
}
}
\item{m}{integer: \strong{default} = nrow(y)

\eqn{{m}}{m}, the number of subfunctions associated with \eqn{{F}\left({x}\right)}{F(x)}.


}
\item{n}{integer: \strong{default} = nrow(x)

\eqn{{n}}{n}, the number of variables.


}
\item{nclin}{integer: \strong{default} = nrow(a)

\eqn{{n}_{{L}}}{n_L}, the number of general linear constraints.


}
\item{ncnln}{integer: \strong{default} = nrow(cjac)

\eqn{{n}_{{N}}}{n_N}, the number of nonlinear constraints.


}
}

\details{
R interface to the NAG Fortran routine E04USF.}

\value{

\item{ITER}{integer

The number of major iterations performed.


}
\item{ISTATE}{integer array

The status of the constraints in the QP working set at the point returned 
in x. The significance of each possible value of \eqn{istate[{j}]}{istate[j]} 
is as follows:


}
\item{C}{double array

If \eqn{ncnln > {0}}{ncnln > 0}, \eqn{c[{i}]}{c[i]} contains the value 
of the \eqn{{i}}{i}th nonlinear constraint function \eqn{{c}_{{i}}}{c_i} 
at the final iterate for \eqn{{i}={1} \ldots ncnln}{i=1  . . .  ncnln}.


}
\item{CJAC}{double array

If \eqn{ncnln > {0}}{ncnln > 0}, cjac contains the Jacobian matrix of the 
nonlinear constraint functions at the final iterate, i.e., \eqn{cjac[{i}, 
{j}]}{cjac[i, j]} contains the partial derivative of the \eqn{{i}}{i}th 
constraint function with respect to the \eqn{{j}}{j}th variable for \eqn{{j}={1} 
\ldots n}{j=1  . . .  n} for \eqn{{i}={1} \ldots ncnln}{i=1  . . .  ncnln}. (See 
the discussion of argument cjac under confun.)


}
\item{F}{double array

\eqn{f[{i}]}{f[i]} contains the value of the \eqn{{i}}{i}th function \eqn{{f}_{{i}}}{f_i} 
at the final iterate for \eqn{{i}={1} \ldots m}{i=1  . . .  m}.


}
\item{FJAC}{double array

The Jacobian matrix of the functions \eqn{{f}_{{1}} , {f}_{{2}} , \ldots , {f}_{{m}}}{f_1 , f_2 ,  . . .  , f_m} 
at the final iterate, i.e., \eqn{fjac[{i}, {j}]}{fjac[i, j]} contains the 
partial derivative of the \eqn{{i}}{i}th function with respect to the \eqn{{j}}{j}th 
variable for \eqn{{j}={1} \ldots n}{j=1  . . .  n} for \eqn{{i}={1} \ldots m}{i=1  . . .  m}. 
(See also the discussion of argument fjac under objfun.)


}
\item{CLAMDA}{double array

The values of the QP multipliers from the last QP subproblem. \eqn{clamda[{j}]}{clamda[j]} 
should be non-negative if \eqn{istate[{j}] = {1}}{istate[j] = 1} and non-positive 
if \eqn{istate[{j}] = {2}}{istate[j] = 2}.


}
\item{OBJF}{double

The value of the objective function at the final iterate.


}
\item{R}{double array

If \eqn{hessian=\code{NO}}{hessian=NO}, r contains the upper triangular 
Cholesky factor \eqn{{R}}{R} of \eqn{{Q}^{T}\tilde{{H}}{Q}}{Q^TtildeHQ}, 
an estimate of the transformed and reordered Hessian of the Lagrangian 
at \eqn{{x}}{x} (see eqn6). If \eqn{hessian=\code{YES}}{hessian=YES}, 
r contains the upper triangular Cholesky factor \eqn{{R}}{R} of \eqn{{H}}{H}, 
the approximate (untransformed) Hessian of the Lagrangian, with the variables 
in the natural order.


}
\item{X}{double array

The final estimate of the solution.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04usf.pdf}
}

\author{NAG}

\examples{

optlist <- list()

ifail <- 0
confun = function(mode, ncnln, n, needc, x, cjac, 
    nstate) {
    ldcj <- nrow(cjac)
    
    c <- as.matrix(mat.or.vec(ncnln, 1))
    
    if (nstate == 1) {
        
        cjac <- as.matrix(mat.or.vec(ncnln, n))
        
    }
    
    if (needc[1] > 0) {
        
        if (mode == 0 || mode == 2) {
            
            c[1] <- -0.09 - x[1] \%*\% x[2] + 0.49 \%*\% x[2]
            
        }
        if (mode == 1 || mode == 2) {
            
            cjac[1, 1] <- -x[2]
            
            cjac[1, 2] <- -x[1] + 0.49
            
        }
    }
    list(MODE = as.integer(mode), C = as.matrix(c), CJAC = as.matrix(cjac))
}
objfun = function(mode, m, n, needfi, x, fjac, nstate) {
    ldfj <- nrow(fjac)
    
    f <- as.matrix(mat.or.vec(m, 1))
    a <- matrix(c(8, 8, 10, 10, 10, 10, 12, 12, 12, 12, 14, 14, 
        14, 16, 16, 16, 18, 18, 20, 20, 20, 22, 22, 22, 24, 24, 
        24, 26, 26, 26, 28, 28, 30, 30, 30, 32, 32, 34, 36, 36, 
        38, 38, 40, 42), nrow = 1, ncol = 44, byrow = TRUE)
    for (i in c(1:m)) {
        temp <- exp(-x[2] \%*\% (a[i] - 8))
        
        if (mode == 0 || mode == 2) {
            
            f[i] <- x[1] + (0.49 - x[1]) \%*\% temp
            
        }
        if (mode == 1 || mode == 2) {
            
            fjac[i, 1] <- 1 - temp
            
            fjac[i, 2] <- -(0.49 - x[1]) \%*\% (a[i] - 8) \%*\% temp
            
        }
    }
    list(MODE = as.integer(mode), F = as.matrix(f), FJAC = as.matrix(fjac))
}

a <- matrix(c(1, 1), nrow = 1, ncol = 2, byrow = TRUE)



bl <- matrix(c(0.4, -4, 1, 0), nrow = 4, ncol = 1, 
    byrow = TRUE)



bu <- matrix(c(1e+25, 1e+25, 1e+25, 1e+25), nrow = 4, 
    ncol = 1, byrow = TRUE)



y <- matrix(c(0.49, 0.49, 0.48, 0.47, 0.48, 0.47, 
    0.46, 0.46, 0.45, 0.43, 0.45, 0.43, 0.43, 0.44, 0.43, 0.43, 
    0.46, 0.45, 0.42, 0.42, 0.43, 0.41, 0.41, 0.4, 0.42, 0.4, 
    0.4, 0.41, 0.4, 0.41, 0.41, 0.4, 0.4, 0.4, 0.38, 0.41, 0.4, 
    0.4, 0.41, 0.38, 0.4, 0.4, 0.39, 0.39), nrow = 44, ncol = 1, 
    byrow = TRUE)



istate <- as.matrix(mat.or.vec(4, 1))

cjac <- matrix(c(0, 0), nrow = 1, ncol = 2, byrow = TRUE)



fjac <- matrix(c(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0), nrow = 44, ncol = 2, byrow = TRUE)



clamda <- as.matrix(mat.or.vec(4, 1))

r <- matrix(c(0, 0, 0, 0), nrow = 2, ncol = 2, byrow = TRUE)



x <- matrix(c(0.4, 0), nrow = 2, ncol = 1, byrow = TRUE)



e04us(a, bl, bu, y, confun, objfun, istate, cjac, 
    fjac, clamda, r, x, optlist) 

}

\keyword{math}

\keyword{optimize}
\concept{constraint}
\concept{derivative}
\concept{minimum}
\concept{nonlinear}
\concept{QP}
\concept{sequential}
\concept{square}
\concept{sum}
