
\name{e04yc}
\alias{e04yc}

\title{e04yc: Covariance matrix for nonlinear least squares problem (unconstrained)}

\description{
e04yc returns estimates of elements of the variance-covariance matrix 
of the estimated regression coefficients for a nonlinear least squares 
problem. The estimates are derived from the Jacobian of the function \eqn{{f}\left({x}\right)}{f(x)} 
at the solution.

This function may be used following any one of the nonlinear least squares 
functions e04fc e04fy e04gb e04gy e04gd e04gz e04he e04hy.

}

\usage{
e04yc(job, m, fsumsq, s, v,
      n = nrow(s))
}

\arguments{

\item{job}{integer

Which elements of \eqn{{C}}{C} are returned as follows: 

\eqn{job = { - {1}}}{job =  - 1}: The \eqn{{n}}{n} by \eqn{{n}}{n} symmetric 
matrix \eqn{{C}}{C} is returned.

\eqn{job = {0}}{job = 0}: The diagonal elements of \eqn{{C}}{C} are returned.

\eqn{job 
> {0}}{job > 0}: The elements of column job of \eqn{{C}}{C} are returned.




}
\item{m}{integer

The number \eqn{{m}}{m} of observations (residuals \eqn{{f}_{{i}}\left({x}\right)}{f_i(x)}).


}
\item{fsumsq}{double

The sum of squares of the residuals, \eqn{{F}\left(\bar{{x}}\right)}{F(barx)}, 
at the solution \eqn{\bar{{x}}}{barx}, as returned by the nonlinear least 
squares function.


}
\item{s}{double array

The \eqn{{n}}{n} singular values of the Jacobian as returned by the nonlinear 
least squares function. See the Description in Fortran library documentation 
for information on supplying s following one of the easy-to-use functions.


}
\item{v}{double array

The \eqn{{n}}{n} by \eqn{{n}}{n} right-hand orthogonal matrix (the right 
singular vectors) of \eqn{{J}}{J} as returned by the nonlinear least squares 
function. See the Description in Fortran library documentation for information 
on supplying v following one of the easy-to-use functions.


}
\item{n}{integer: \strong{default} = nrow(s)

The number \eqn{{n}}{n} of variables \eqn{\left({x}_{{j}}\right)}{(x_j)}.


}
}

\details{
R interface to the NAG Fortran routine E04YCF.}

\value{

\item{V}{double array

If \eqn{job \geq {0}}{job  >=  0}, v is unchanged.


}
\item{CJ}{double array

If \eqn{job = {0}}{job = 0}, cj returns the \eqn{{n}}{n} diagonal elements 
of \eqn{{C}}{C}.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/E04/e04ycf.pdf}
}

\author{NAG}

\examples{

ifail <- 0

job <- 0

m <- 15

fsumsq <- 0.00821487730657898

s <- matrix(c(4.096503460741, 1.59495793805472, 0.0612584931217495), 
    nrow = 3, ncol = 1, byrow = TRUE)



v <- matrix(c(0.935395908691802, 0.352951220949886, 
    -0.0214459700788422, -0.259228425671719, 0.643234592093676, 
    -0.72045116618536, -0.240489328924174, 0.679466478322564, 
    0.693173995119214), nrow = 3, ncol = 3, byrow = TRUE)



e04yc(job, m, fsumsq, s, v) 

}

\keyword{math}

\keyword{optimize}
\concept{covariance}
\concept{least squares}
\concept{nonlinear}
