
\name{g02ab}
\alias{g02ab}

\title{g02ab: Computes the nearest correlation matrix to a real square matrix, augmented 
g02aa to incorporate weights and bounds}

\description{
g02ab computes the nearest correlation matrix, in the Frobenius norm or 
weighted Frobenius norm, and optionally with bounds on the eigenvalues, 
to a given square, input matrix.

}

\usage{
g02ab(g, opt, alpha, w,
      n = nrow(w),
      errtol = 0.0,
      maxits = 0,
      maxit = 0)
}

\arguments{

\item{g}{double array

\eqn{{G}}{G}, the initial matrix.


}
\item{opt}{string

Indicates the problem to be solved. 

\eqn{opt=\mathtt{'A'}}{opt='A'}: The lower bound problem is solved.

\eqn{opt=\mathtt{'W'}}{opt='W'}: The weighted norm problem is solved.

\eqn{opt=\mathtt{'B'}}{opt='B'}: Both problems are solved.




}
\item{alpha}{double

The value of \eqn{{\alpha}}{\alpha}.


}
\item{w}{double array

The square roots of the diagonal elements of \eqn{{W}}{W}, that is the 
diagonal of \eqn{{W}^{\frac{{1}}{{2}}}}{W^(1)/(2)}.


}
\item{n}{integer: \strong{default} = nrow(w)

The size of the matrix \eqn{{G}}{G}.


}
\item{errtol}{double: \strong{default} = 0.0

The termination tolerance for the Newton iteration. If \eqn{errtol \leq {0.0}}{errtol  <=  0.0} 
then \eqn{{n} \times \sqrt{machine precision}}{n \times sqrt(machine precision)} 
is used.


}
\item{maxits}{integer: \strong{default} = 0

Specifies the maximum number of iterations to be used by the iterative 
scheme to solve the linear algebraic equations at each Newton step.


}
\item{maxit}{integer: \strong{default} = 0

Specifies the maximum number of Newton iterations.


}
}

\details{
R interface to the NAG Fortran routine G02ABF.}

\value{

\item{G}{double array

A symmetric matrix \eqn{\frac{{1}}{{2}}\left({G} + {G}^{T}\right)}{(1)/(2)(G + G^T)} 
with the diagonal set to \eqn{{I}}{I}.


}
\item{W}{double array

If \eqn{opt=\mathtt{'W'}, \mathtt{'B'}}{opt='W', 'B'}, the array is scaled 
so \eqn{\mathrm{max}\left({W}_{{i}}\right) = {1}}{max(W_i) = 1} for \eqn{{i}={1} 
\ldots {n}}{i=1  . . .  n}.


}
\item{X}{double array

Contains the nearest correlation matrix.


}
\item{ITER}{integer

The number of Newton steps taken.


}
\item{FEVAL}{integer

The number of function evaluations of the dual problem.


}
\item{NRMGRD}{double

The norm of the gradient of the last Newton step.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/G02/g02abf.pdf}
}

\author{NAG}

\examples{

ifail <- 0

opt <- "b"

alpha <- 0.02

g <- matrix(c(2, -1, 0, 0, -1, 2, -1, 0, 0, -1, 2, 
    -1, 0, 0, -1, 2), nrow = 4, ncol = 4, byrow = TRUE)



w <- matrix(c(100, 20, 20, 20), nrow = 4, ncol = 1, 
    byrow = TRUE)



errtol <- 1e-07

maxits <- 200

maxit <- 10

ans <- g02ab(g, opt, alpha, w)

if (ifail == 0) {
    
    writeLines(sprintf("\n Nearest Correlation Matrix\n", 
        "\n"))
    
    
    x <- ans$X
    
    print(x) 
    
    iter <- ans$ITER
    
    writeLines(sprintf("\n Number of Newton steps taken: \%d\n", 
        iter))
    
    
    feval <- ans$FEVAL
    
    writeLines(sprintf(" Number of function evaluations: \%d\n", 
        feval))
    
    
    alpha <- ans$ALPHA
    
    writeLines(sprintf(" \n\n Alpha: \%30.3f\n", 
        alpha))
    
    
} 

}

\keyword{math}
\concept{Correlation Matrix}
