
\name{s17dc}
\alias{s17dc}

\title{s17dc: Bessel functions Y_nu + a(z), real a  >=  0, complex z, nu = 0 , 1 , 2 ,  . . . }

\description{
s17dc returns a sequence of values for the Bessel functions \eqn{{Y}_{{{\nu} 
+ {n}}}\left({z}\right)}{Y_\nu + n(z)} for complex \eqn{{z}}{z}, non-negative 
\eqn{{\nu}}{\nu} and \eqn{{n} = {0} , {1} , \ldots , {N} - {1}}{n = 0 , 1 ,  . . .  , N - 1}, 
with an option for exponential scaling.

}

\usage{
s17dc(fnu, z, n, scal)
}

\arguments{

\item{fnu}{double

\eqn{{\nu}}{\nu}, the order of the first member of the sequence of functions.


}
\item{z}{complex

\eqn{{z}}{z}, the argument of the functions.


}
\item{n}{integer

\eqn{{N}}{N}, the number of members required in the sequence \eqn{{Y}_{{\nu}}\left({z}\right) 
, {Y}_{{{\nu} + {1}}}\left({z}\right) , \ldots , {Y}_{{{\nu} + {N} - {1}}}\left({z}\right)}{Y_\nu(z) , Y_\nu + 1(z) ,  . . .  , Y_\nu + N - 1(z)}.


}
\item{scal}{string

The scaling option. 

\eqn{scal=\mathtt{'U'}}{scal='U'}: The results are returned unscaled.

\eqn{scal=\mathtt{'S'}}{scal='S'}: The results are returned scaled by the 
factor \eqn{{e}^{{ - \mathrm{abs}(\mathrm{Im}\left({z}\right))}}}{e^ - abs(Im(z))}.




}
}

\details{
R interface to the NAG Fortran routine S17DCF.}

\value{

\item{CY}{complex array

The \eqn{{N}}{N} required function values: \eqn{cy[{i}]}{cy[i]} contains 
\eqn{{Y}_{{{\nu} + {i} - {1}}}\left({z}\right)}{Y_\nu + i - 1(z)} for \eqn{{i}={1} 
\ldots {N}}{i=1  . . .  N}.


}
\item{NZ}{integer

The number of components of cy that are set to zero due to underflow. The 
positions of such components in the array cy are arbitrary.


}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/S/s17dcf.pdf}
}

\author{NAG}

\examples{

ifail<-0

fnu<-0

z<-complex(1,0.3,0.4)

n<-2

scal<-'U'

s17dc(fnu,z,n,scal)

}

\keyword{math}
\concept{Bessel}
