
\name{s17dg}
\alias{s17dg}

\title{s17dg: Airy functions Ai(z) and Ai'(z), complex z}

\description{
s17dg returns the value of the Airy function \eqn{\mathrm{Ai}\left({z}\right)}{Ai(z)} 
or its derivative \eqn{{Ai}'\left({z}\right)}{Ai'(z)} for complex \eqn{{z}}{z}, 
with an option for exponential scaling.

}

\usage{
s17dg(deriv, z, scal)
}

\arguments{

\item{deriv}{string

Specifies whether the function or its derivative is required. 

If \eqn{deriv=\mathtt{'F'}}{deriv='F'}, \eqn{\mathrm{Ai}\left({z}\right)}{Ai(z)} 
is returned.

If \eqn{deriv=\mathtt{'D'}}{deriv='D'}, \eqn{{Ai}'\left({z}\right)}{Ai'(z)} 
is returned.




}
\item{z}{complex

The argument \eqn{{z}}{z} of the function.


}
\item{scal}{string

The scaling option. 

\eqn{scal=\mathtt{'U'}}{scal='U'}: The result is returned unscaled.

\eqn{scal=\mathtt{'S'}}{scal='S'}: The result is returned scaled by the 
factor \eqn{{e}^{{{2}{z}  \sqrt{{z}} / {3}}}}{e^2z  sqrt(z) / 3}.




}
}

\details{
R interface to the NAG Fortran routine S17DGF.}

\value{

\item{AI}{complex

The required function or derivative value.


}
\item{NZ}{integer

Indicates whether or not ai is set to zero due to underflow. This can only 
occur when \eqn{scal=\mathtt{'U'}}{scal='U'}. 

\eqn{nz = {0}}{nz = 0}: ai is not set to zero.

\eqn{nz = {1}}{nz = 1}: ai is set to zero.




}
\item{IFAIL}{integer

\eqn{\mathrm{ifail} =0}{ifail =0} unless the function detects an error 
or a warning has been flagged (see the Errors section in Fortran library 
documentation).


}
}

\references{
\url{http://www.nag.co.uk/numeric/FL/nagdoc_fl23/pdf/S/s17dgf.pdf}
}

\author{NAG}

\examples{

ifail<-0

deriv<-'F'

z<-complex(1,0.3,0.4)

scal<-'U'

s17dg(deriv,z,scal)

}

\keyword{math}
\concept{airy}
\concept{Ai}
