
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04gzf_LSFUN2(CONST Integer *M, CONST Integer *N,
                                   CONST double XC[], double FVEC[],
                                   double FJAC[], CONST Integer *LDFJAC,
                                   Integer *E04GZF_IUSER,
                                   double E04GZF_RUSER[]);

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04GZF_clean_up(double *X, double *W);

SEXP WE04GZF(SEXP M, SEXP N, SEXP LSFUN2, SEXP X, SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04GZF_IUSER[4];
  double  E04GZF_RUSER[1];
  SEXP    ans, names;

/*initialization for input variables*/
  int     M_integer;
  if (nag_extractInteger(M, &M_integer))
    {
      error("Integer extraction error for %s", "M");
    }
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  double  *X_doubleMatrix = 0;

/*initialization for dimension variables*/
  Integer LW_integer;
  if (N_integer > 1)
    {
      LW_integer = 8*N_integer+2*N_integer*N_integer+2*M_integer*N_integer+3*
                   M_integer;
    }
  else
    {
      if (N_integer == 1)
        {
          LW_integer = 11+5*M_integer;
        }
    }

/*initialization for output variables*/
  double  FSUMSQ_double;

/*allocation general variables*/
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }

/*initialization for workspace variables*/
  double  *W_doubleMatrix = 0;

/*allocation for workspace variables*/
  Integer W_dim1 = LW_integer;
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "W");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04GZF_clean_up(X_doubleMatrix, W_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04GZF_IUSER[0]);
  nag_convert_p2i(LSFUN2, &E04GZF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04gzf_(&M_integer, &N_integer, e04gzf_LSFUN2, X_doubleMatrix,
          &FSUMSQ_double, W_doubleMatrix, &LW_integer, E04GZF_IUSER,
          E04GZF_RUSER,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04GZF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 3));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_d2r(1, &FSUMSQ_double));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 3));
  SET_STRING_ELT(names, 0, mkChar("X"));
  SET_STRING_ELT(names, 1, mkChar("FSUMSQ"));
  SET_STRING_ELT(names, 2, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04GZF_clean_up(X_doubleMatrix, W_doubleMatrix);

  return(ans);
}

void NAG_CALL e04gzf_LSFUN2(CONST Integer *M, CONST Integer *N,
                            CONST double XC[], double FVEC[], double FJAC[],
                            CONST Integer *LDFJAC,
                            Integer *E04GZF_IUSER,
                            double E04GZF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04GZF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04GZF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang5(
            funct,
            nag_fi2r(1, M),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_fi2r(1, LDFJAC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleMatrixFromList("FVEC", 1, *M, 1, result, FVEC);
  (void) nag_getDoubleMatrixFromList("FJAC", 1, *LDFJAC, *N, result, FJAC);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04GZF_clean_up(double *X, double *W)
{
  if (X) free(X);
  if (W) free(W);
  return;
}
