
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04hdf_FUNCT(Integer *IFLAG, CONST Integer *N,
                                  CONST double XC[], double *FC, double GC[],
                                  Integer IW[], CONST Integer *LIW,
                                  double W[],
                                  CONST Integer *LW);
extern void NAG_CALL e04hdf_H(Integer *IFLAG, CONST Integer *N,
                              CONST double XC[], double FHESL[],
                              CONST Integer *LH, double FHESD[],
                              Integer IW[], CONST Integer *LIW, double W[],
                              CONST Integer *LW);

#ifdef __cplusplus
}
#endif

/*Declaration of global IUSER for this routine*/
Integer E04HDF_IUSER[6];

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04HDF_clean_up(double *X, double *G, double *HESL, double *HESD,
                     Integer *IW,
                     double *W);

SEXP WE04HDF(SEXP N, SEXP FUNCT, SEXP H, SEXP X, SEXP LH, SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  double  *X_doubleMatrix = 0;
  int     LH_integer;
  if (nag_extractInteger(LH, &LH_integer))
    {
      error("Integer extraction error for %s", "LH");
    }

/*initialization for dimension variables*/
  Integer LIW_integer = 1;
  Integer LW_integer = 5*N_integer;

/*initialization for output variables*/
  double  *G_doubleMatrix = 0;
  double  *HESL_doubleMatrix = 0;
  double  *HESD_doubleMatrix = 0;

/*allocation general variables*/
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }
  Integer G_dim1 = N_integer;
  if (!(G_doubleMatrix = (double *) malloc(G_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "G");
    }
  Integer HESL_dim1 = LH_integer;
  if (!(HESL_doubleMatrix = (double *) malloc(HESL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "HESL");
    }
  Integer HESD_dim1 = N_integer;
  if (!(HESD_doubleMatrix = (double *) malloc(HESD_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "HESD");
    }

/*initialization for communication variables*/
  Integer *IW_integerMatrix = 0;
  double  *W_doubleMatrix = 0;

/*allocation communication variables*/
  Integer IW_dim1 = LIW_integer;
  if (!(IW_integerMatrix = (Integer *) malloc(IW_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IW");
    }
  Integer W_dim1 = LW_integer;
  if (!(W_doubleMatrix = (double *) malloc(W_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "W");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, X_dim1, 1, X, X_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "X", nrows(X), X_dim1);
        }
      else
        {
          E04HDF_clean_up(X_doubleMatrix, G_doubleMatrix, HESL_doubleMatrix,
                          HESD_doubleMatrix, IW_integerMatrix,
                          W_doubleMatrix);
          error("Array extraction error for %s", "X");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04HDF_IUSER[0]);
  nag_convert_p2i(FUNCT, &E04HDF_IUSER[2]);
  nag_convert_p2i(H, &E04HDF_IUSER[4]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04hdf_(&N_integer, e04hdf_FUNCT, e04hdf_H, X_doubleMatrix, G_doubleMatrix,
          HESL_doubleMatrix, &LH_integer, HESD_doubleMatrix, IW_integerMatrix,
          &LIW_integer, W_doubleMatrix, &LW_integer,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04HDF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 4));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, G_dim1, 1, G_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, HESL_dim1, 1, HESL_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, HESD_dim1, 1, HESD_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 4));
  SET_STRING_ELT(names, 0, mkChar("G"));
  SET_STRING_ELT(names, 1, mkChar("HESL"));
  SET_STRING_ELT(names, 2, mkChar("HESD"));
  SET_STRING_ELT(names, 3, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04HDF_clean_up(X_doubleMatrix, G_doubleMatrix, HESL_doubleMatrix,
                  HESD_doubleMatrix, IW_integerMatrix,
                  W_doubleMatrix);

  return(ans);
}

void NAG_CALL e04hdf_FUNCT(Integer *IFLAG, CONST Integer *N, CONST double XC[],
                           double *FC, double GC[], Integer IW[],
                           CONST Integer *LIW, double W[],
                           CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04HDF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04HDF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang4(
            funct,
            nag_fi2r(1, IFLAG),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("IFLAG", result, IFLAG);
  (void) nag_getDoubleScalarFromList("FC", result, FC);
  (void) nag_getDoubleMatrixFromList("GC", 1, *N, 1, result, GC);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e04hdf_H(Integer *IFLAG, CONST Integer *N, CONST double XC[],
                       double FHESL[], CONST Integer *LH, double FHESD[],
                       Integer IW[],
                       CONST Integer *LIW, double W[],
                       CONST Integer *LW)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04HDF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04HDF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang6(
            funct,
            nag_fi2r(1, IFLAG),
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, XC),
            nag_fi2r(1, LH),
            nag_md2r(1, *N, 1, FHESD)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("IFLAG", result, IFLAG);
  (void) nag_getDoubleMatrixFromList("FHESL", 1, *LH, 1, result, FHESL);
  (void) nag_getDoubleMatrixFromList("FHESD", 1, *N, 1, result, FHESD);

/* Release R_fcall */
  UNPROTECT(2);
}

void E04HDF_clean_up(double *X, double *G, double *HESL, double *HESD,
                     Integer *IW,
                     double *W)
{
  if (X) free(X);
  if (G) free(G);
  if (HESL) free(HESL);
  if (HESD) free(HESD);
  if (IW) free(IW);
  if (W) free(W);
  return;
}
