
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e04nka_QPHX(CONST Integer *NSTATE, CONST Integer *NCOLH,
                                 CONST double X[], double HX[],
                                 Integer *E04NKF_IUSER,
                                 double E04NKF_RUSER[]);

#ifdef __cplusplus
}
#endif
int wE04NKF_handle_options(SEXP OPTLIST, Integer LWSAV_logicalArray[],
                           Integer IWSAV_integerMatrix[],
                           double RWSAV_doubleMatrix[])
;

/*Declaration of global IUSER for this routine*/
Integer E04NKF_IUSER[4];

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E04NKA_clean_up(double *A, Integer *HA, Integer *KA, double *BL,
                     double *BU, char *NAMES, char *CRNAME, double *XS,
                     Integer *ISTATE,
                     double *CLAMDA, Integer *IZ, double *Z, Integer *LWSAV,
                     Integer *IWSAV,
                     double *RWSAV);

SEXP WE04NKF(SEXP N, SEXP M, SEXP NNZ, SEXP IOBJ, SEXP NCOLH, SEXP QPHX,
             SEXP A, SEXP HA, SEXP KA, SEXP BL, SEXP BU, SEXP START,
             SEXP NAMES, SEXP NNAME,
             SEXP CRNAME, SEXP NS, SEXP XS, SEXP ISTATE, SEXP LENIZ, SEXP LENZ,
             SEXP OPTLIST,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E04NKF_IUSER[4];
  double  E04NKF_RUSER[1];
  SEXP    ans, names;
  Integer nadv, iflag;

  /* Initialize the option arrays */
  Integer IFAIL_initializer_integer = 1;
  char    err_msg[LEN_ERR_MSG];
  char    RNAME_char[6] = "E04NKA";
  Integer RNAME_length = 6;

/*Declare dimension variables for option initializer*/
  Integer LCWSAV_integer;
  if ((strcmp(RNAME_char,
              "E04UFF") == 0) || (strcmp(RNAME_char, "E04UFA") == 0))
    {
      LCWSAV_integer = 5;
    }
  else
    {
      LCWSAV_integer = 1;
    }

  Integer LLWSAV_integer;
  if ((strcmp(RNAME_char,
              "E04NKF") == 0) || (strcmp(RNAME_char, "E04NKA") == 0))
    {
      LLWSAV_integer = 20;
    }
  else
    {
      if ((strcmp(RNAME_char,
                  "E04UGF") == 0) || (strcmp(RNAME_char, "E04UGA") == 0))
        {
          LLWSAV_integer = 20;
        }
      else
        {
          LLWSAV_integer = 120;
        }
    }

  Integer LIWSAV_integer;
  if ((strcmp(RNAME_char,
              "E04NKF") == 0) || (strcmp(RNAME_char, "E04NKA") == 0))
    {
      LIWSAV_integer = 380;
    }
  else
    {
      if ((strcmp(RNAME_char,
                  "E04UGF") == 0) || (strcmp(RNAME_char, "E04UGA") == 0))
        {
          LIWSAV_integer = 550;
        }
      else
        {
          LIWSAV_integer = 610;
        }
    }

  Integer LRWSAV_integer;
  if ((strcmp(RNAME_char,
              "E04NKF") == 0) || (strcmp(RNAME_char, "E04NKA") == 0))
    {
      LRWSAV_integer = 285;
    }
  else
    {
      if ((strcmp(RNAME_char,
                  "E04UGF") == 0) || (strcmp(RNAME_char, "E04UGA") == 0))
        {
          LRWSAV_integer = 550;
        }
      else
        {
          LRWSAV_integer = 475;
        }
    }

/*Declare the parameters for the option initializer*/
  char    *CWSAV_charArray = 0;
  Integer *LWSAV_logicalArray = 0;
  Integer *IWSAV_integerMatrix = 0;
  double  *RWSAV_doubleMatrix = 0;
  Integer CWSAV_dim1 = LCWSAV_integer;
  if (!(CWSAV_charArray = (char *) malloc(CWSAV_dim1*80*sizeof(char))))
    {
      error("Memory allocation error for %s", "CWSAV");
    }
  Integer LWSAV_dim1 = LLWSAV_integer;
  if (!(LWSAV_logicalArray = (Integer *) malloc(LWSAV_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "LWSAV");
    }
  Integer IWSAV_dim1 = LIWSAV_integer;
  if (!(IWSAV_integerMatrix = (Integer *) malloc(IWSAV_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IWSAV");
    }
  Integer RWSAV_dim1 = LRWSAV_integer;
  if (!(RWSAV_doubleMatrix = (double *) malloc(RWSAV_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "RWSAV");
    }

  nag_initializeIntegerMatrix(IWSAV_dim1, 1, IWSAV_integerMatrix);
  nag_initializeDoubleMatrix(RWSAV_dim1, 1, RWSAV_doubleMatrix);
  e04wbf_(RNAME_char, CWSAV_charArray, &LCWSAV_integer, LWSAV_logicalArray,
          &LLWSAV_integer, IWSAV_integerMatrix, &LIWSAV_integer,
          RWSAV_doubleMatrix,
          &LRWSAV_integer, &IFAIL_initializer_integer, RNAME_length,
          80);

  if (IFAIL_initializer_integer != 0)
    {
      sprintf(err_msg, "E04WBF failed with IFAIL = %ld\n",
              (long) IFAIL_initializer_integer);
      error(err_msg);
    } /* Handle options */
  int     unit_num;

  unit_num =
    wE04NKF_handle_options(OPTLIST, LWSAV_logicalArray, IWSAV_integerMatrix,
                           RWSAV_doubleMatrix);
  if (unit_num < 0)
    {
      /* Error opening output file */
      sprintf(
        err_msg,
        "Error opening monitoring file for output, check value for `monitoring file` entry in `optlist`");
      error(err_msg);
    }
  else if (unit_num > 0)
    {
      /* We are using an output file, so need to handle the advisory channel */

      /* Remember the current value */
      iflag = 0;
      x04abf_(&iflag, &nadv);

      /* Set the advisory channel to the output file. Need to do this, otherwise E04NKF sends output to both the
         advisory channel and the output file */
      iflag = 1;
      x04abf_(&iflag, &unit_num);
    }

/*initialization for input variables*/
  int        N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  int        M_integer;
  if (nag_extractInteger(M, &M_integer))
    {
      error("Integer extraction error for %s", "M");
    }
  int        NNZ_integer;
  if (nag_extractInteger(NNZ, &NNZ_integer))
    {
      error("Integer extraction error for %s", "NNZ");
    }
  int        IOBJ_integer;
  if (nag_extractInteger(IOBJ, &IOBJ_integer))
    {
      error("Integer extraction error for %s", "IOBJ");
    }
  int        NCOLH_integer;
  if (nag_extractInteger(NCOLH, &NCOLH_integer))
    {
      error("Integer extraction error for %s", "NCOLH");
    }
  double     *A_doubleMatrix = 0;
  Integer    *HA_integerMatrix = 0;
  Integer    *KA_integerMatrix = 0;
  double     *BL_doubleMatrix = 0;
  double     *BU_doubleMatrix = 0;
  const char *START_char = CHAR(STRING_ELT(START, 0));
  Integer    START_length = 1;
  char       *NAMES_charArray = 0;
  int        NNAME_integer;
  if (nag_extractInteger(NNAME, &NNAME_integer))
    {
      error("Integer extraction error for %s", "NNAME");
    }
  char       *CRNAME_charArray = 0;
  int        NS_integer;
  if (nag_extractInteger(NS, &NS_integer))
    {
      error("Integer extraction error for %s", "NS");
    }
  double     *XS_doubleMatrix = 0;
  Integer    *ISTATE_integerMatrix = 0;
  int        LENIZ_integer;
  if (nag_extractInteger(LENIZ, &LENIZ_integer))
    {
      error("Integer extraction error for %s", "LENIZ");
    }
  int        LENZ_integer;
  if (nag_extractInteger(LENZ, &LENZ_integer))
    {
      error("Integer extraction error for %s", "LENZ");
    }

/*initialization for output variables*/
  int        MINIZ_integer;
  int        MINZ_integer;
  int        NINF_integer;
  double     SINF_double;
  double     OBJ_double;
  double     *CLAMDA_doubleMatrix = 0;

/*allocation general variables*/
  Integer    A_dim1 = NNZ_integer;
  if (!(A_doubleMatrix = (double *) malloc(A_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "A");
    }
  Integer    HA_dim1 = NNZ_integer;
  if (!(HA_integerMatrix = (Integer *) malloc(HA_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "HA");
    }
  Integer    KA_dim1 = N_integer+1;
  if (!(KA_integerMatrix = (Integer *) malloc(KA_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "KA");
    }
  Integer    BL_dim1 = N_integer+M_integer;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BL");
    }
  Integer    BU_dim1 = N_integer+M_integer;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BU");
    }
  Integer    NAMES_dim1 = 5;
  if (!(NAMES_charArray = (char *) malloc(NAMES_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "NAMES");
    }
  Integer    CRNAME_dim1 = NNAME_integer;
  if (!(CRNAME_charArray = (char *) malloc(CRNAME_dim1*8*sizeof(char))))
    {
      error("Memory allocation error for %s", "CRNAME");
    }
  Integer    XS_dim1 = N_integer+M_integer;
  if (!(XS_doubleMatrix = (double *) malloc(XS_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "XS");
    }
  Integer    ISTATE_dim1 = N_integer+M_integer;
  if (!(ISTATE_integerMatrix = (Integer *) malloc(ISTATE_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "ISTATE");
    }
  Integer    CLAMDA_dim1 = N_integer+M_integer;
  if (!(CLAMDA_doubleMatrix = (double *) malloc(CLAMDA_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "CLAMDA");
    }

/*initialization for workspace variables*/
  Integer    *IZ_integerMatrix = 0;
  double     *Z_doubleMatrix = 0;

/*allocation for workspace variables*/
  Integer    IZ_dim1 = LENIZ_integer;
  if (!(IZ_integerMatrix = (Integer *) malloc(IZ_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "IZ");
    }
  Integer    Z_dim1 = LENZ_integer;
  if (!(Z_doubleMatrix = (double *) malloc(Z_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "Z");
    }

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, A_dim1, 1, A, A_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "A", nrows(A), A_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "A");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, HA_dim1, 1, HA, HA_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "HA", nrows(HA), HA_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "HA");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, KA_dim1, 1, KA, KA_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "KA", nrows(KA), KA_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "KA");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BL_dim1, 1, BL, BL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BL", nrows(BL), BL_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "BL");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BU_dim1, 1, BU, BU_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BU", nrows(BU), BU_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "BU");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractCharVector(NAMES_dim1, NAMES, NAMES_charArray, 8)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "NAMES", nrows(NAMES), NAMES_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "NAMES");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractCharVector(CRNAME_dim1, CRNAME, CRNAME_charArray, 8)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "CRNAME", nrows(CRNAME), CRNAME_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "CRNAME");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, XS_dim1, 1, XS, XS_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "XS", nrows(XS), XS_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "XS");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, ISTATE_dim1, 1, ISTATE,
                                  ISTATE_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "ISTATE", nrows(ISTATE), ISTATE_dim1);
        }
      else
        {
          E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                          BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                          CRNAME_charArray,
                          XS_doubleMatrix, ISTATE_integerMatrix,
                          CLAMDA_doubleMatrix,
                          IZ_integerMatrix, Z_doubleMatrix, LWSAV_logicalArray,
                          IWSAV_integerMatrix,
                          RWSAV_doubleMatrix);
          error("Array extraction error for %s", "ISTATE");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E04NKF_IUSER[0]);
  nag_convert_p2i(QPHX, &E04NKF_IUSER[2]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e04nka_(&N_integer, &M_integer, &NNZ_integer, &IOBJ_integer, &NCOLH_integer,
          e04nka_QPHX, A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
          BL_doubleMatrix, BU_doubleMatrix, START_char, NAMES_charArray,
          &NNAME_integer,
          CRNAME_charArray, &NS_integer, XS_doubleMatrix, ISTATE_integerMatrix,
          &MINIZ_integer, &MINZ_integer, &NINF_integer, &SINF_double,
          &OBJ_double,
          CLAMDA_doubleMatrix, IZ_integerMatrix, &LENIZ_integer,
          Z_doubleMatrix,
          &LENZ_integer, E04NKF_IUSER, E04NKF_RUSER, LWSAV_logicalArray,
          IWSAV_integerMatrix, RWSAV_doubleMatrix, &IFAIL_integer,
          START_length, 8,
          8);

  /* Close the monitoring file if one opened */

  if (unit_num > 0)
    {
      IFAIL_integer = 1;
      x04adf_(&unit_num, &IFAIL_integer);

      /* Set the advisory channel back to its previous value */
      iflag = 1;
      x04abf_(&iflag, &nadv);
    }

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E04NKF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 10));
  SET_VECTOR_ELT(ans, 0, nag_i2r(1, &NS_integer));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, XS_dim1, 1, XS_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_mi2r(1, ISTATE_dim1, 1, ISTATE_integerMatrix));
  SET_VECTOR_ELT(ans, 3, nag_i2r(1, &MINIZ_integer));
  SET_VECTOR_ELT(ans, 4, nag_i2r(1, &MINZ_integer));
  SET_VECTOR_ELT(ans, 5, nag_i2r(1, &NINF_integer));
  SET_VECTOR_ELT(ans, 6, nag_d2r(1, &SINF_double));
  SET_VECTOR_ELT(ans, 7, nag_d2r(1, &OBJ_double));
  SET_VECTOR_ELT(ans, 8, nag_md2r(1, CLAMDA_dim1, 1, CLAMDA_doubleMatrix));
  SET_VECTOR_ELT(ans, 9, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 10));
  SET_STRING_ELT(names, 0, mkChar("NS"));
  SET_STRING_ELT(names, 1, mkChar("XS"));
  SET_STRING_ELT(names, 2, mkChar("ISTATE"));
  SET_STRING_ELT(names, 3, mkChar("MINIZ"));
  SET_STRING_ELT(names, 4, mkChar("MINZ"));
  SET_STRING_ELT(names, 5, mkChar("NINF"));
  SET_STRING_ELT(names, 6, mkChar("SINF"));
  SET_STRING_ELT(names, 7, mkChar("OBJ"));
  SET_STRING_ELT(names, 8, mkChar("CLAMDA"));
  SET_STRING_ELT(names, 9, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E04NKA_clean_up(A_doubleMatrix, HA_integerMatrix, KA_integerMatrix,
                  BL_doubleMatrix, BU_doubleMatrix, NAMES_charArray,
                  CRNAME_charArray,
                  XS_doubleMatrix, ISTATE_integerMatrix, CLAMDA_doubleMatrix,
                  IZ_integerMatrix,
                  Z_doubleMatrix, LWSAV_logicalArray, IWSAV_integerMatrix,
                  RWSAV_doubleMatrix);

  return(ans);
}

void NAG_CALL e04nka_QPHX(CONST Integer *NSTATE, CONST Integer *NCOLH,
                          CONST double X[], double HX[], Integer *E04NKF_IUSER,
                          double E04NKF_RUSER[])
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E04NKF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E04NKF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang4(
            funct,
            nag_fi2r(1, NSTATE),
            nag_fi2r(1, NCOLH),
            nag_md2r(1, *NCOLH, 1, X)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleMatrixFromList("HX", 1, *NCOLH, 1, result, HX);

/* Release R_fcall */
  UNPROTECT(2);
}

int wE04NKF_handle_options(SEXP OPTLIST, Integer LWSAV_logicalArray[],
                           Integer IWSAV_integerMatrix[],
                           double RWSAV_doubleMatrix[])
{
  SEXP       names, tentry;
  char       err_msg[LEN_ERR_MSG];
  int        i, ival;
  Integer    INFORM_setter_integer, STR_length;
  double     dval;
  const char *this_name;
  char       STR_char[100];
  int        unit_num = 0, mode = 1;

  /* No options specified */
  if (!isNewList(OPTLIST))
    {
      return 0;
    }

  /* Get the list of names from the list */
  PROTECT(names = getAttrib(OPTLIST, R_NamesSymbol));
  for (i = 0; i < length(OPTLIST); i++)
    {
      this_name = CHAR(STRING_ELT(names, i));

      /* Only interested in the first element, which is what asReal should
         convert to a double */
      PROTECT(tentry = VECTOR_ELT(OPTLIST, i));
      if (!isString(tentry))
        {
          dval = asReal(tentry);
          if (!R_FINITE(dval)) dval = 0.0;
          ival = (int) dval;
        }
      else
        {
          dval = -1.0;
          ival = -1;
        }

      STR_char[0] = '\0';
      if (!strcmp(this_name, "list"))
        {
          if (ival)
            {
              sprintf(STR_char, "list");
            }
          else
            {
              sprintf(STR_char, "nolist");
            }
        }
      else if (!strcmp(this_name, "Check Frequency"))
        {
          sprintf(STR_char, "Check Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Option"))
        {
          sprintf(STR_char, "Crash Option = %d", ival);
        }
      else if (!strcmp(this_name, "Crash Tolerance"))
        {
          sprintf(STR_char, "Crash Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Expand Frequency"))
        {
          sprintf(STR_char, "Expand Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Factorization Frequency"))
        {
          sprintf(STR_char, "Factorization Frequency = %d", ival);
        }
      else if (!strcmp(this_name, "Feasibility Tolerance"))
        {
          sprintf(STR_char, "Feasibility Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Infinite Bound Size"))
        {
          sprintf(STR_char, "Infinite Bound Size = %g", dval);
        }
      else if (!strcmp(this_name, "Infinite Step Size"))
        {
          sprintf(STR_char, "Infinite Step Size = %g", dval);
        }
      else if (!strcmp(this_name, "Iteration Limit"))
        {
          sprintf(STR_char, "Iteration Limit = %d", ival);
        }
      else if (!strcmp(this_name, "LU Singularity Tolerance"))
        {
          sprintf(STR_char, "LU Singularity Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Monitoring File")) /* Can't have mode 0 as this is reading mode only */
        {
          if (mode == 0) mode = 1;
          /* Use unit_num 7 by default */
          if (unit_num == 0) unit_num = 7;
          /* Open up a file for output (either write or append) */
          Integer IFAIL_write_integer = 1;

          x04acf_(&unit_num,
                  strncpy(STR_char, CHAR(STRING_ELT(tentry,
                                                    0)),
                          100), &mode, &IFAIL_write_integer, 100);
          if (IFAIL_write_integer != 0)
            {
              unit_num = -1;
              break;
            }
          sprintf(STR_char, "monitoring file = %d", unit_num);
        }
      else if (!strcmp(this_name, "Optimality Tolerance"))
        {
          sprintf(STR_char, "Optimality Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Partial Price"))
        {
          sprintf(STR_char, "Partial Price = %d", ival);
        }
      else if (!strcmp(this_name, "Pivot Tolerance"))
        {
          sprintf(STR_char, "Pivot Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Print Level"))
        {
          sprintf(STR_char, "Print Level = %d", ival);
        }
      else if (!strcmp(this_name, "Rank Tolerance"))
        {
          sprintf(STR_char, "Rank Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Scale Option"))
        {
          sprintf(STR_char, "Scale Option = %d", ival);
        }
      else if (!strcmp(this_name, "Scale Tolerance"))
        {
          sprintf(STR_char, "Scale Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Superbasics Limit"))
        {
          sprintf(STR_char, "Superbasics Limit = %d", ival);
        }

      /* All other options are ignored (this includes the valid ones: cold start and warm start) */

      UNPROTECT(1);

      /* Call the option setting routine */
      if (STR_char[0])
        {
          INFORM_setter_integer = 1;
          STR_length = strlen(STR_char);
          e04nma_(STR_char, LWSAV_logicalArray, IWSAV_integerMatrix,
                  RWSAV_doubleMatrix, &INFORM_setter_integer,
                  STR_length);

          if (INFORM_setter_integer != 0)
            {
              sprintf(err_msg, "E04NMA failed with IFAIL = %ld\n",
                      (long) INFORM_setter_integer);
              error(err_msg);
            }
        }
    }
  UNPROTECT(1);
  return unit_num;
}

void E04NKA_clean_up(double *A, Integer *HA, Integer *KA, double *BL,
                     double *BU, char *NAMES, char *CRNAME, double *XS,
                     Integer *ISTATE,
                     double *CLAMDA, Integer *IZ, double *Z, Integer *LWSAV,
                     Integer *IWSAV,
                     double *RWSAV)
{
  if (A) free(A);
  if (HA) free(HA);
  if (KA) free(KA);
  if (BL) free(BL);
  if (BU) free(BU);
  if (NAMES) free(NAMES);
  if (CRNAME) free(CRNAME);
  if (XS) free(XS);
  if (ISTATE) free(ISTATE);
  if (CLAMDA) free(CLAMDA);
  if (IZ) free(IZ);
  if (Z) free(Z);
  if (LWSAV) free(LWSAV);
  if (IWSAV) free(IWSAV);
  if (RWSAV) free(RWSAV);
  return;
}
