
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif
extern void NAG_CALL e05jbf_OBJFUN(CONST Integer *N, CONST double X[],
                                   double *F, CONST Integer *NSTATE,
                                   Integer *E05JBF_IUSER,
                                   double E05JBF_RUSER[],
                                   Integer *INFORM);
extern void NAG_CALL e05jbf_MONIT(CONST Integer *N, CONST Integer *NCALL,
                                  CONST double XBEST[], CONST Integer ICOUNT[],
                                  CONST Integer *NINIT,
                                  CONST double LIST[], CONST Integer NUMPTS[],
                                  CONST Integer INITPT[],
                                  CONST Integer *NBASKT, CONST double XBASKT[],
                                  CONST double BOXL[],
                                  CONST double BOXU[], CONST Integer *NSTATE,
                                  Integer *E05JBF_IUSER,
                                  double E05JBF_RUSER[],
                                  Integer *INFORM);

#ifdef __cplusplus
}
#endif
int wE05JBF_handle_options(SEXP OPTLIST, double COMM_doubleMatrix[],
                           CONST Integer *LCOMM_integer)
;

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void E05JBF_clean_up(double *BL, double *BU, double *LIST, Integer *NUMPTS,
                     Integer *INITPT, double *X,
                     double *COMM);

SEXP WE05JBF(SEXP N, SEXP OBJFUN, SEXP IBOUND, SEXP IINIT, SEXP BL, SEXP BU,
             SEXP SDLIST, SEXP LIST, SEXP NUMPTS, SEXP INITPT, SEXP MONIT,
             SEXP OPTLIST,
             SEXP RHO)
{
  Integer IFAIL_integer, IFAIL_EXTRACTION_integer;
  Integer E05JBF_IUSER[6];
  double  E05JBF_RUSER[1];
  SEXP    ans, names;
  Integer nadv, iflag;

  /* Initialize the option arrays */
  Integer IFAIL_initializer_integer = 1;
  char    err_msg[LEN_ERR_MSG];

/*Declare dimension variables for option initializer*/
  Integer LCOMM_integer = 100;

/*Declare the parameters for the option initializer*/
  double  *COMM_doubleMatrix = 0;
  Integer COMM_dim1 = LCOMM_integer;
  if (!(COMM_doubleMatrix = (double *) malloc(COMM_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "COMM");
    }
  int     N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  nag_initializeDoubleMatrix(COMM_dim1, 1, COMM_doubleMatrix);
  e05jaf_(&N_integer, COMM_doubleMatrix, &LCOMM_integer,
          &IFAIL_initializer_integer);

  if (IFAIL_initializer_integer != 0)
    {
      sprintf(err_msg, "E05JAF failed with IFAIL = %ld\n",
              (long) IFAIL_initializer_integer);
      error(err_msg);
    } /* Handle options */
  int     unit_num;

  unit_num = wE05JBF_handle_options(OPTLIST, COMM_doubleMatrix, &LCOMM_integer);
  if (unit_num < 0)
    {
      /* Error opening output file */
      sprintf(
        err_msg,
        "Error opening monitoring file for output, check value for `monitoring file` entry in `optlist`");
      error(err_msg);
    }
  else if (unit_num > 0)
    {
      /* We are using an output file, so need to handle the advisory channel */

      /* Remember the current value */
      iflag = 0;
      x04abf_(&iflag, &nadv);

      /* Set the advisory channel to the output file. Need to do this, otherwise E05JBF sends output to both the
         advisory channel and the output file */
      iflag = 1;
      x04abf_(&iflag, &unit_num);
    }

/*initialization for input variables*/
  int     IBOUND_integer;
  if (nag_extractInteger(IBOUND, &IBOUND_integer))
    {
      error("Integer extraction error for %s", "IBOUND");
    }
  int     IINIT_integer;
  if (nag_extractInteger(IINIT, &IINIT_integer))
    {
      error("Integer extraction error for %s", "IINIT");
    }
  double  *BL_doubleMatrix = 0;
  double  *BU_doubleMatrix = 0;
  double  *LIST_doubleMatrix = 0;
  Integer *NUMPTS_integerMatrix = 0;
  Integer *INITPT_integerMatrix = 0;

/*initialization for dimension variables*/
  Integer SDLIST_integer;
  Integer NUMPTS_dim1 = N_integer;
  if (!(NUMPTS_integerMatrix = (Integer *) malloc(NUMPTS_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "NUMPTS");
    }
  if ((IINIT_integer != 2))
    {
      SDLIST_integer = 3;
    }
  else
    {
      if ((IINIT_integer == 2))
        {
          SDLIST_integer = 192;
        }
      else
        {
          if ((IINIT_integer == 3))
            {
              SDLIST_integer = nag_getMaxValueIntegerVector(
                N_integer,
                NUMPTS_integerMatrix);
            }
        }
    }

/*initialization for output variables*/
  double  *X_doubleMatrix = 0;
  double  OBJ_double;

/*allocation general variables*/
  Integer BL_dim1 = N_integer;
  if (!(BL_doubleMatrix = (double *) malloc(BL_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BL");
    }
  Integer BU_dim1 = N_integer;
  if (!(BU_doubleMatrix = (double *) malloc(BU_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "BU");
    }
  Integer LIST_dim1 = N_integer;
  Integer LIST_dim2 = SDLIST_integer;
  if (!(LIST_doubleMatrix =
          (double *) malloc(LIST_dim1*LIST_dim2*sizeof(double))))
    {
      error("Memory allocation error for %s", "LIST");
    }
  Integer INITPT_dim1 = N_integer;
  if (!(INITPT_integerMatrix = (Integer *) malloc(INITPT_dim1*sizeof(Integer))))
    {
      error("Memory allocation error for %s", "INITPT");
    }
  Integer X_dim1 = N_integer;
  if (!(X_doubleMatrix = (double *) malloc(X_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "X");
    }

/*allocation for dimension variables*/

/*extraction for input variables*/
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BL_dim1, 1, BL, BL_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BL", nrows(BL), BL_dim1);
        }
      else
        {
          E05JBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, LIST_doubleMatrix,
                          NUMPTS_integerMatrix, INITPT_integerMatrix,
                          X_doubleMatrix,
                          COMM_doubleMatrix);
          error("Array extraction error for %s", "BL");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, BU_dim1, 1, BU, BU_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "BU", nrows(BU), BU_dim1);
        }
      else
        {
          E05JBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, LIST_doubleMatrix,
                          NUMPTS_integerMatrix, INITPT_integerMatrix,
                          X_doubleMatrix,
                          COMM_doubleMatrix);
          error("Array extraction error for %s", "BU");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractDoubleMatrix(1, LIST_dim1, LIST_dim2, LIST,
                                 LIST_doubleMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "LIST", nrows(LIST), LIST_dim1);
        }
      else
        {
          E05JBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, LIST_doubleMatrix,
                          NUMPTS_integerMatrix, INITPT_integerMatrix,
                          X_doubleMatrix,
                          COMM_doubleMatrix);
          error("Array extraction error for %s", "LIST");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, NUMPTS_dim1, 1, NUMPTS,
                                  NUMPTS_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "NUMPTS", nrows(NUMPTS), NUMPTS_dim1);
        }
      else
        {
          E05JBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, LIST_doubleMatrix,
                          NUMPTS_integerMatrix, INITPT_integerMatrix,
                          X_doubleMatrix,
                          COMM_doubleMatrix);
          error("Array extraction error for %s", "NUMPTS");
        }
    }
  if ((IFAIL_EXTRACTION_integer =
         nag_extractIntegerMatrix(1, INITPT_dim1, 1, INITPT,
                                  INITPT_integerMatrix)))
    {
      if ((IFAIL_EXTRACTION_integer == 3) || (IFAIL_EXTRACTION_integer == 4))
        {
          warning(
            "Dimensions mismatch for %s, matrix is of size %d and the expected size is %d",
            "INITPT", nrows(INITPT), INITPT_dim1);
        }
      else
        {
          E05JBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, LIST_doubleMatrix,
                          NUMPTS_integerMatrix, INITPT_integerMatrix,
                          X_doubleMatrix,
                          COMM_doubleMatrix);
          error("Array extraction error for %s", "INITPT");
        }
    }

/* Perform some initial checks */
  if (!isEnvironment(RHO))
    {
      error("rho must be an environment");
    }

  /* Convert function pointer funct and rho into a couple of integers and store in IUSER */
  nag_convert_p2i(RHO, &E05JBF_IUSER[0]);
  nag_convert_p2i(OBJFUN, &E05JBF_IUSER[2]);
  nag_convert_p2i(MONIT, &E05JBF_IUSER[4]);

  /* Call the NAG routine */
  IFAIL_integer = 1;
  e05jbf_(&N_integer, e05jbf_OBJFUN, &IBOUND_integer, &IINIT_integer,
          BL_doubleMatrix, BU_doubleMatrix, &SDLIST_integer, LIST_doubleMatrix,
          NUMPTS_integerMatrix, INITPT_integerMatrix, e05jbf_MONIT,
          X_doubleMatrix,
          &OBJ_double, COMM_doubleMatrix, &LCOMM_integer, E05JBF_IUSER,
          E05JBF_RUSER,
          &IFAIL_integer);

  /* Close the monitoring file if one opened */

  if (unit_num > 0)
    {
      IFAIL_integer = 1;
      x04adf_(&unit_num, &IFAIL_integer);

      /* Set the advisory channel back to its previous value */
      iflag = 1;
      x04abf_(&iflag, &nadv);
    }

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  E05JBF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 8));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, BL_dim1, 1, BL_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_md2r(1, BU_dim1, 1, BU_doubleMatrix));
  SET_VECTOR_ELT(ans, 2, nag_md2r(1, LIST_dim1, LIST_dim2, LIST_doubleMatrix));
  SET_VECTOR_ELT(ans, 3, nag_mi2r(1, NUMPTS_dim1, 1, NUMPTS_integerMatrix));
  SET_VECTOR_ELT(ans, 4, nag_mi2r(1, INITPT_dim1, 1, INITPT_integerMatrix));
  SET_VECTOR_ELT(ans, 5, nag_md2r(1, X_dim1, 1, X_doubleMatrix));
  SET_VECTOR_ELT(ans, 6, nag_d2r(1, &OBJ_double));
  SET_VECTOR_ELT(ans, 7, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 8));
  SET_STRING_ELT(names, 0, mkChar("BL"));
  SET_STRING_ELT(names, 1, mkChar("BU"));
  SET_STRING_ELT(names, 2, mkChar("LIST"));
  SET_STRING_ELT(names, 3, mkChar("NUMPTS"));
  SET_STRING_ELT(names, 4, mkChar("INITPT"));
  SET_STRING_ELT(names, 5, mkChar("X"));
  SET_STRING_ELT(names, 6, mkChar("OBJ"));
  SET_STRING_ELT(names, 7, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  E05JBF_clean_up(BL_doubleMatrix, BU_doubleMatrix, LIST_doubleMatrix,
                  NUMPTS_integerMatrix, INITPT_integerMatrix, X_doubleMatrix,
                  COMM_doubleMatrix);

  return(ans);
}

void NAG_CALL e05jbf_OBJFUN(CONST Integer *N, CONST double X[], double *F,
                            CONST Integer *NSTATE, Integer *E05JBF_IUSER,
                            double E05JBF_RUSER[],
                            Integer *INFORM)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E05JBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E05JBF_IUSER[2], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang4(
            funct,
            nag_fi2r(1, N),
            nag_md2r(1, *N, 1, X),
            nag_fi2r(1, NSTATE)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getDoubleScalarFromList("F", result, F);
  (void) nag_getIntFromList("INFORM", result, INFORM);

/* Release R_fcall */
  UNPROTECT(2);
}

void NAG_CALL e05jbf_MONIT(CONST Integer *N, CONST Integer *NCALL,
                           CONST double XBEST[], CONST Integer ICOUNT[],
                           CONST Integer *NINIT,
                           CONST double LIST[], CONST Integer NUMPTS[],
                           CONST Integer INITPT[],
                           CONST Integer *NBASKT, CONST double XBASKT[],
                           CONST double BOXL[],
                           CONST double BOXU[], CONST Integer *NSTATE,
                           Integer *E05JBF_IUSER,
                           double E05JBF_RUSER[],
                           Integer *INFORM)
{
  SEXP funct, rho, R_fcall, result;
/* Get the pointer to the R environment */
  nag_convert_i2p(&E05JBF_IUSER[0], &rho); /* Get the pointer to the R function */
  nag_convert_i2p(&E05JBF_IUSER[4], &funct);

/* Give the arguments to the function */
  PROTECT(R_fcall = lang14(
            funct,
            nag_fi2r(1, N),
            nag_fi2r(1, NCALL),
            nag_md2r(1, *N, 1, XBEST),
            nag_fi2r(6, ICOUNT),
            nag_fi2r(1, NINIT),
            nag_md2r(1, *N, *NINIT, LIST),
            nag_fi2r(*N, NUMPTS),
            nag_fi2r(*N, INITPT),
            nag_fi2r(1, NBASKT),
            nag_md2r(1, *N, *NBASKT, XBASKT),
            nag_md2r(1, *N, 1, BOXL),
            nag_md2r(1, *N, 1, BOXU),
            nag_fi2r(1, NSTATE)));

/* Get the results from evaluating the function in R */
  PROTECT(result = eval(R_fcall, rho));
/* put the results returned by function into C variables */
  (void) nag_getIntFromList("INFORM", result, INFORM);

/* Release R_fcall */
  UNPROTECT(2);
}

int wE05JBF_handle_options(SEXP OPTLIST, double COMM_doubleMatrix[],
                           CONST Integer *LCOMM_integer)
{
  SEXP       names, tentry;
  char       err_msg[LEN_ERR_MSG];
  int        i, ival;
  Integer    IFAIL_setter_integer, OPTSTR_length;
  double     dval;
  const char *this_name;
  char       OPTSTR_char[100];
  int        unit_num = 0;

  /* No options specified */
  if (!isNewList(OPTLIST))
    {
      return 0;
    }

  /* Get the list of names from the list */
  PROTECT(names = getAttrib(OPTLIST, R_NamesSymbol));
  for (i = 0; i < length(OPTLIST); i++)
    {
      this_name = CHAR(STRING_ELT(names, i));

      /* Only interested in the first element, which is what asReal should
         convert to a double */
      PROTECT(tentry = VECTOR_ELT(OPTLIST, i));
      if (!isString(tentry))
        {
          dval = asReal(tentry);
          if (!R_FINITE(dval)) dval = 0.0;
          ival = (int) dval;
        }
      else
        {
          dval = -1.0;
          ival = -1;
        }

      OPTSTR_char[0] = '\0';
      if (!strcmp(this_name, "list"))
        {
          if (ival)
            {
              sprintf(OPTSTR_char, "list");
            }
          else
            {
              sprintf(OPTSTR_char, "nolist");
            }
        }
      else if (!strcmp(this_name, "Function Evaluations Limit"))
        {
          sprintf(OPTSTR_char, "Function Evaluations Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Infinite Bound Size"))
        {
          sprintf(OPTSTR_char, "Infinite Bound Size = %g", dval);
        }
      else if (!strcmp(this_name, "Local Searches Limit"))
        {
          sprintf(OPTSTR_char, "Local Searches Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Local Searches Tolerance"))
        {
          sprintf(OPTSTR_char, "Local Searches Tolerance = %g", dval);
        }
      else if (!strcmp(this_name, "Splits Limit"))
        {
          sprintf(OPTSTR_char, "Splits Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Static Limit"))
        {
          sprintf(OPTSTR_char, "Static Limit = %d", ival);
        }
      else if (!strcmp(this_name, "Target Objective Error"))
        {
          sprintf(OPTSTR_char, "Target Objective Error = %g", dval);
        }
      else if (!strcmp(this_name, "Target Objective Safeguard"))
        {
          sprintf(OPTSTR_char, "Target Objective Safeguard = %g", dval);
        }
      else if (!strcmp(this_name, "Target Objective Value"))
        {
          sprintf(OPTSTR_char, "Target Objective Value = %g", dval);
        }

      /* All other options are ignored (this includes the valid ones: cold start and warm start) */

      UNPROTECT(1);

      /* Call the option setting routine */
      if (OPTSTR_char[0])
        {
          IFAIL_setter_integer = 1;
          OPTSTR_length = strlen(OPTSTR_char);
          e05jdf_(OPTSTR_char, COMM_doubleMatrix, LCOMM_integer,
                  &IFAIL_setter_integer,
                  OPTSTR_length);

          if (IFAIL_setter_integer != 0)
            {
              sprintf(err_msg, "E05JDF failed with IFAIL = %ld\n",
                      (long) IFAIL_setter_integer);
              error(err_msg);
            }
        }
    }
  UNPROTECT(1);
  return unit_num;
}

void E05JBF_clean_up(double *BL, double *BU, double *LIST, Integer *NUMPTS,
                     Integer *INITPT, double *X,
                     double *COMM)
{
  if (BL) free(BL);
  if (BU) free(BU);
  if (LIST) free(LIST);
  if (NUMPTS) free(NUMPTS);
  if (INITPT) free(INITPT);
  if (X) free(X);
  if (COMM) free(COMM);
  return;
}
