
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void S17DCF_clean_up(Complex *CY, Complex *CWRK);

SEXP WS17DCF(SEXP FNU, SEXP Z, SEXP N, SEXP SCAL)
{
  Integer    IFAIL_integer;
  SEXP       ans, names;

/*initialization for input variables*/
  double     FNU_double;
  if (nag_extractDouble(FNU, &FNU_double))
    {
      error("Double extraction error for %s", "FNU");
    }
  Complex    Z_complex;
  if (nag_extractComplex(Z, &Z_complex))
    {
      error("Complex extraction error for %s", "Z");
    }
  int        N_integer;
  if (nag_extractInteger(N, &N_integer))
    {
      error("Integer extraction error for %s", "N");
    }
  const char *SCAL_char = CHAR(STRING_ELT(SCAL, 0));
  Integer    SCAL_length = 1;

/*initialization for output variables*/
  Complex    *CY = 0;
  int        NZ_integer;

/*allocation general variables*/
  Integer    CY_dim1 = N_integer;
  if (!(CY = (Complex *) malloc(CY_dim1*sizeof(Complex))))
    {
      error("Memory allocation error for %s", "CY");
    }

/*initialization for workspace variables*/
  Complex    *CWRK = 0;

/*allocation for workspace variables*/
  Integer    CWRK_dim1 = N_integer;
  if (!(CWRK = (Complex *) malloc(CWRK_dim1*sizeof(Complex))))
    {
      error("Memory allocation error for %s", "CWRK");
    }

/*extraction for input variables*/

  /* Call the NAG routine */
  IFAIL_integer = 1;
  s17dcf_(&FNU_double, &Z_complex, &N_integer, SCAL_char, CY, &NZ_integer,
          CWRK, &IFAIL_integer,
          SCAL_length);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  S17DCF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 3));
  SET_VECTOR_ELT(ans, 0, nag_cplx2r(CY_dim1, CY));
  SET_VECTOR_ELT(ans, 1, nag_i2r(1, &NZ_integer));
  SET_VECTOR_ELT(ans, 2, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 3));
  SET_STRING_ELT(names, 0, mkChar("CY"));
  SET_STRING_ELT(names, 1, mkChar("NZ"));
  SET_STRING_ELT(names, 2, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  S17DCF_clean_up(CY, CWRK);

  return(ans);
}

void S17DCF_clean_up(Complex *CY, Complex *CWRK)
{
  if (CY) free(CY);
  if (CWRK) free(CWRK);
  return;
}
