
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void S17DHF_clean_up(void);

SEXP WS17DHF(SEXP DERIV, SEXP Z, SEXP SCAL)
{
  Integer    IFAIL_integer;
  SEXP       ans, names;

/*initialization for input variables*/
  const char *DERIV_char = CHAR(STRING_ELT(DERIV, 0));
  Integer    DERIV_length = 1;
  Complex    Z_complex;
  if (nag_extractComplex(Z, &Z_complex))
    {
      error("Complex extraction error for %s", "Z");
    }
  const char *SCAL_char = CHAR(STRING_ELT(SCAL, 0));
  Integer    SCAL_length = 1;

/*initialization for output variables*/
  Complex    BI_complex;

/*allocation general variables*/

/*extraction for input variables*/

  /* Call the NAG routine */
  IFAIL_integer = 1;
  s17dhf_(DERIV_char, &Z_complex, SCAL_char, &BI_complex, &IFAIL_integer,
          DERIV_length,
          SCAL_length);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  S17DHF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 2));
  SET_VECTOR_ELT(ans, 0, nag_cplx2r(1, &BI_complex));
  SET_VECTOR_ELT(ans, 1, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 2));
  SET_STRING_ELT(names, 0, mkChar("BI"));
  SET_STRING_ELT(names, 1, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  S17DHF_clean_up();

  return(ans);
}

void S17DHF_clean_up(void)
{
  return;
}
