
#include <R.h>
#include <Rinternals.h>
#include "nag_rwrapper_utils.h"
#include "nagmk23.h"
#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

/* Length of array to hold error messages, no checks are made to ensure this is large enough */
#define LEN_ERR_MSG 100

void S22AAF_clean_up(double *P);

SEXP WS22AAF(SEXP MODE, SEXP X, SEXP M, SEXP NL)
{
  Integer IFAIL_integer;
  SEXP    ans, names;

/*initialization for input variables*/
  int     MODE_integer;
  if (nag_extractInteger(MODE, &MODE_integer))
    {
      error("Integer extraction error for %s", "MODE");
    }
  double  X_double;
  if (nag_extractDouble(X, &X_double))
    {
      error("Double extraction error for %s", "X");
    }
  int     M_integer;
  if (nag_extractInteger(M, &M_integer))
    {
      error("Integer extraction error for %s", "M");
    }
  int     NL_integer;
  if (nag_extractInteger(NL, &NL_integer))
    {
      error("Integer extraction error for %s", "NL");
    }

/*initialization for output variables*/
  double  *P_doubleMatrix = 0;

/*allocation general variables*/
  Integer P_dim1 = NL_integer+1;
  if (!(P_doubleMatrix = (double *) malloc(P_dim1*sizeof(double))))
    {
      error("Memory allocation error for %s", "P");
    }

/*extraction for input variables*/

  /* Call the NAG routine */
  IFAIL_integer = 1;
  s22aaf_(&MODE_integer, &X_double, &M_integer, &NL_integer, P_doubleMatrix,
          &IFAIL_integer);

  /* Error returns from NAG routine */
  if (IFAIL_integer != 0)
    {
      warning(
        "ABNORMAL EXIT from NAG library routine  S22AAF, : IFAIL = %ld\n",
        (long) IFAIL_integer);
    }

/* Get the list of results */
  PROTECT(ans = allocVector(VECSXP, 2));
  SET_VECTOR_ELT(ans, 0, nag_md2r(1, P_dim1, 1, P_doubleMatrix));
  SET_VECTOR_ELT(ans, 1, nag_i2r(1, &IFAIL_integer));
  /* Get names for the list of results */
  PROTECT(names = allocVector(STRSXP, 2));
  SET_STRING_ELT(names, 0, mkChar("P"));
  SET_STRING_ELT(names, 1, mkChar("IFAIL"));
  setAttrib(ans, R_NamesSymbol, names);
/* Unprotect the ans and names objects */
  UNPROTECT(2);

/*Call the clean-up routine*/
  S22AAF_clean_up(P_doubleMatrix);

  return(ans);
}

void S22AAF_clean_up(double *P)
{
  if (P) free(P);
  return;
}
